@@ -2,10 +2,12 @@
 Classes for storing configuration by namespace, and reloading configuration
 while files change.
 """
+import functools
 import logging
 import os
 import time
 from collections import namedtuple
+import weakref
 
 from staticconf import errors
 
@@ -42,6 +44,7 @@ class ConfigMap(object):
     def __len__(self):
         return len(self.data)
 
+
 class ConfigNamespace(object):
     """A configuration namespace, which contains the list of value proxies
     and configuration values.
@@ -50,14 +53,17 @@ class ConfigNamespace(object):
     def __init__(self, name):
         self.name = name
         self.configuration_values = {}
-        self.value_proxies = []
+        self.value_proxies = weakref.WeakValueDictionary()
+
+    def get_name(self):
+        return self.name
 
     def get_value_proxies(self):
-        return self.value_proxies
+        return self.value_proxies.values()
 
     def register_proxy(self, proxy):
         """Register a new value proxy in this namespace."""
-        self.value_proxies.append(proxy)
+        self.value_proxies[id(proxy)] = proxy
 
     def apply_config_data(self, config_data, error_on_unknown, error_on_dupe):
         """Validate, check for duplicates, and update the config."""
@@ -71,12 +77,14 @@ class ConfigNamespace(object):
     def get_config_values(self):
         return self.configuration_values
 
+    def get_known_keys(self):
+        return set(vproxy.config_key for vproxy in self.get_value_proxies())
+
     def validate_keys(self, config_data, error_on_unknown):
         """Raise an exception if error_on_unknown is true, and keys contains
         a key which is not defined in a registeredValueProxy.
         """
-        known_keys = set(vproxy.config_key for vproxy in self.value_proxies)
-        unknown = remove_by_keys(config_data, known_keys)
+        unknown = remove_by_keys(config_data, self.get_known_keys())
         if not unknown:
             return
 
@@ -89,6 +97,9 @@ class ConfigNamespace(object):
         args = config_data, self.configuration_values, error_on_duplicate
         return has_duplicate_keys(*args)
 
+    def get(self, item, default=None):
+        return self.configuration_values.get(item, default)
+
     def __getitem__(self, item):
         return self.configuration_values[item]
 
@@ -103,7 +114,10 @@ class ConfigNamespace(object):
 
     def _reset(self):
         self.clear()
-        self.value_proxies[:] = []
+        self.value_proxies.clear()
+
+    def __str__(self):
+        return "%s(%s)" % (type(self).__name__, self.name)
 
 
 config_key_descriptions = {}
@@ -244,6 +258,9 @@ class ConfigurationWatcher(object):
         self.reloader()
         return config_dict
 
+    def get_reloader(self):
+        return self.reloader
+
 
 class ReloadCallbackChain(object):
     """This object can be used as a convenient way of adding and removing
@@ -265,3 +282,27 @@ class ReloadCallbackChain(object):
         reload(name=self.namespace, all_names=self.all_names)
         for callback in self.callbacks.itervalues():
             callback()
+
+
+class ConfigFacade(object):
+    """A facade around a ConfigurationWatcher and a ReloadCallbackChain.
+    """
+
+    def __init__(self, watcher):
+        self.watcher = watcher
+        self.callback_chain = watcher.get_reloader()
+
+    @classmethod
+    def load(cls, filename, namespace, loader_func, min_interval=0):
+        loader = functools.partial(loader_func, filename, namespace=namespace)
+        reloader = ReloadCallbackChain(namespace=namespace)
+        watcher = ConfigurationWatcher(
+            loader, filename, min_interval=min_interval, reloader=reloader)
+        loader()
+        return cls(watcher)
+
+    def add_callback(self, identifier, callback):
+        self.callback_chain.add(identifier, callback)
+
+    def reload_if_changed(self, force=False):
+        self.watcher.reload_if_changed(force=force)
